<?php

/* DB BACKUP
   Run as Cron Job (See docs)
------------------------------------------------------------------------------*/

define('PATH', substr(__dir__, 0, -13) . '/');
define('PARENT', 1);
define('CRON_RUN', 1);

// ENABLE DEBUGGER
include(PATH . 'control/system/constants.php');
if (version_compare(PHP_VERSION, '8.0.0') >= 0) {
  include(PATH . 'control/lib/tracy/vendor/tracy/tracy/src/tracy.php');
}
use Tracy\Debugger;
if (version_compare(PHP_VERSION, '8.0.0') >= 0) {
  // For development, report everything and enable tracy bar
  Debugger::$showBar = (defined('DEVELOPMENT_ENV') ? true : false);
  Debugger::$strictMode = (defined('DEVELOPMENT_ENV') ? true : false);
  Debugger::$errorTemplate = PATH . 'control/lib/tracy/500.php';
  Debugger::enable(Debugger::DEVELOPMENT, PATH . 'logs');
}

// Session class we don't need here, so just initialise it
include(PATH . 'control/classes/system/class.session.php');
$SSN = new stdclass();

date_default_timezone_set('UTC');
include(PATH . 'control/system/init.php');

// Set limits
$ini_get = ini_get_all();
if (MS_SET_MEM_ALLOCATION_LIMIT && isset($ini_get['memory_limit'])) {
  ini_set('memory_limit', MS_SET_MEM_ALLOCATION_LIMIT);
}
set_time_limit(MS_SET_TIME_OUT_LIMIT);

include(PATH . 'control/classes/mailer/mail-init.php');
include(PATH . 'control/classes/class.backup.php');

if (isset($SETTINGS->bfolder) && $SETTINGS->bfolder && is_dir($SETTINGS->bfolder) && is_writeable($SETTINGS->bfolder)) {
  $filepath         = $SETTINGS->bfolder . '/' . $msg_script33 . '-' . date('dMY', $MSDT->mswTimeStamp()) . '-' . date('His', $MSDT->mswTimeStamp()) . '.gz';
  $BACKUP           = new dbBackup($filepath, true);
  $BACKUP->settings = $SETTINGS;
  $BACKUP->dt       = $MSDT;

  //-----------------------
  // Backup database
  //-----------------------

  $BACKUP->doDump();

  //-------------------------------------------
  // Send emails if there are backup emails..
  //-------------------------------------------

  if (file_exists($filepath) && $SETTINGS->backupEmails) {
    $em_add = '';
    $em     = '';
    if (strpos($SETTINGS->backupEmails, ',') !== false) {
      // Multiple addresses..
      $emails = array_map('trim', explode(',', $SETTINGS->backupEmails));
      // First email is main address..
      if (isset($emails[0])) {
        $em = $emails[0];
        // Now remove it from array..
        unset($emails[0]);
      }
      // Implode additional addresses..
      if (!empty($emails)) {
        $em_add = implode(',', $emails);
      }
    } else {
      // Just main address..
      $em = $SETTINGS->backupEmails;
    }
    if ($em) {
      $MSMAIL->addTag('{HELPDESK}', mswCD($SETTINGS->website));
      $MSMAIL->addTag('{DATE_TIME}', $MSDT->mswDateTimeDisplay($MSDT->mswTimeStamp(), $SETTINGS->dateformat) . ' @ ' . $MSDT->mswDateTimeDisplay($MSDT->mswTimeStamp(), $SETTINGS->timeformat));
      $MSMAIL->addTag('{VERSION}', SCRIPT_VERSION);
      $MSMAIL->addTag('{FILE}', basename($filepath));
      $MSMAIL->addTag('{SCRIPT}', SCRIPT_NAME);
      $MSMAIL->addTag('{SIZE}', mswFSC(@filesize($filepath)));
      $MSMAIL->attachments[$filepath] = basename($filepath);
      $MSMAIL->sendMSMail(array(
        'from_email' => $SETTINGS->email,
        'from_name' => $SETTINGS->website,
        'to_email' => $em,
        'to_name' => $em,
        'subject' => str_replace(array(
          '{website}'
        ), array(
          $SETTINGS->website
        ), $emailSubjects['db-backup']),
        'replyto' => array(
          'name' => $SETTINGS->website,
          'email' => ($SETTINGS->replyto ? $SETTINGS->replyto : $SETTINGS->email)
        ),
        'template' => PATH . 'content/language/' . $SETTINGS->language . '/mail-templates/backup.txt',
        'language' => $SETTINGS->language,
        'alive' => ($em_add ? 'yes' : 'no'),
        'add-emails' => $em_add
      ));
      // Close connection
      $MSMAIL->smtpClose();
      // Remove file..
      try {
        unlink($filepath);
      } catch(Exception $e) {
        Tracy\Debugger::log($e->getMessage());
      } catch(Error $e) {
        Tracy\Debugger::log($e->getMessage());
      }
    }
  }

  echo '[' . date('j F Y @ H:iA') . '] ' . $msg_script32 . mswNL() . str_repeat('-=', 50) . mswNL();
} else {
  echo '"' . $SETTINGS->bfolder . '" folder must exist and be writeable. Please check settings, directory and permissions..';
}
sessNuke();

?>